// Copyright (c) 2001-2002 Rob Kaper <cap@capsi.com>,
//               2001 Erik Bourget <ebourg@cs.mcgill.ca>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// version 2 as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

#ifndef MONOP_ESTATE_H
#define MONOP_ESTATE_H

#include <string>
#include <vector>

#include "gameobject.h"

class CardGroup;
class Estate;
class EstateGroup;
class Game;
class Player;

class Estate : public GameObject
{
public:
	Estate(int id, const std::string name, std::vector<Estate *> *);

	void setEstateGroup(EstateGroup *group) { m_group = group; }
	void setOwner(Player *_p) { m_owner = _p; }
	void setMortgaged(bool _m) { m_mortgaged = _m; }
	void setColor(const std::string color) { m_color = color; }
	void setBgColor(const std::string color) {m_bgColor = color; }
	void setIcon(const std::string &icon);
	std::string icon() const;
	void setPrice(unsigned int _price) { m_price = _price; }
	void setHousePrice(unsigned int price) { m_housePrice = price; }
	void setSellHousePrice(int _price) { m_sellhouseprice = _price; }
	void setMortgagePrice(int _p) { m_mortgageprice = _p; }
	void setUnmortgagePrice(int _p) { m_unmortgageprice = _p; }
	void setTax(int _tax) { m_tax = _tax; }
	void setTaxPercentage(int _tax) { m_taxPercentage = _tax; }
	void setHouses(int _h) { m_houses = _h; }
	void setRent(int _houses, int _m) { m_rent[_houses] = _m; }
	void setMoney(int money);
	int money();
	void addMoney(int money);
	void setPayAmount(int amount);
	int payAmount();
	void setPayTarget(Estate *estate) { m_payTarget = estate; }
	void setPassMoney(int passMoney);
	int passMoney();

	void setToJail(bool toJail);
	bool toJail();
	void setJail(bool jail);
	bool jail();
	void setTakeCardGroup(CardGroup *cardGroup);
	CardGroup *takeCardGroup();

	EstateGroup *group() { return m_group; }
	Player *owner() { return m_owner; }
	bool mortgaged() { return m_mortgaged; }
	const std::string color();
	const std::string bgColor();
	unsigned int price();
	unsigned int housePrice();
	int sellHousePrice() { return m_sellhouseprice; }
	int mortgagePrice() { return m_mortgageprice; }
	int unmortgagePrice() { return m_unmortgageprice; }
	int tax() { return m_tax; }
	int taxPercentage() { return m_taxPercentage; }
	int houses() { return m_houses; }
	int rentByHouses(int _h) { return m_rent[_h]; }
	int rent(Player *player, bool modifier);
	Estate *payTarget() { return m_payTarget; }

	bool canBeOwned();
	bool canToggleMortgage(Player *owner);
	bool canBuyHouses(Player *);
	bool canSellHouses(Player *);
		
	int groupSize(Player *);
	int minHouses();
	int maxHouses();
	bool groupHasBuildings();
	bool groupHasMortgages();
	bool estateGroupIsMonopoly();

private:
	Estate *m_payTarget;
	EstateGroup *m_group;
	Player *m_owner;
	CardGroup *m_takeCardGroup;
	std::vector<Estate *> *m_estates;
	bool m_mortgaged, m_toJail, m_jail;
	int m_money, m_passMoney, m_payAmount, m_sellhouseprice, m_mortgageprice, m_unmortgageprice, m_tax, m_taxPercentage, m_houses, m_rent[6];
	unsigned int m_price, m_housePrice;
	std::string m_color, m_bgColor, m_icon;
};

#endif // MONOP_ESTATE_H
