// Copyright (c) 2002 Rob Kaper <cap@capsi.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// version 2 as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.

#include <iostream>

#include <math++/nodes.h>

#include "estate.h"
#include "estategroup.h"
#include "game.h"
#include "io.h"
#include "player.h"

EstateGroup::EstateGroup(int id, Game *game, std::string name)
	: GameObject(id, name, GameObject::Unknown, game)
{
	m_rentMath = m_rentVar = "";
	m_price = m_housePrice = 0;
}

void EstateGroup::setPrice(unsigned int price)
{
	if (m_price != price)
		m_price = price;
}

unsigned int EstateGroup::price()
{
	return m_price;
}

void EstateGroup::setHousePrice(unsigned int housePrice)
{
	if (m_housePrice != housePrice)
		m_housePrice = housePrice;
}

unsigned int EstateGroup::housePrice()
{
	return m_housePrice;
}

void EstateGroup::setColor(const std::string color)
{
	if (m_color != color)
		m_color = color;
}

const std::string EstateGroup::color()
{
	return m_color;
}

void EstateGroup::setBgColor(const std::string bgColor)
{
	if (m_bgColor != bgColor)
		m_bgColor = bgColor;
}


const std::string EstateGroup::bgColor()
{
	return m_bgColor;
}

void EstateGroup::setRentMath(const std::string rentMath)
{
	if (m_rentMath != rentMath)
		m_rentMath = rentMath;
}

void EstateGroup::setRentVar(const std::string rentVar)
{
	if (m_rentVar != rentVar)
		m_rentVar = rentVar;
}

int EstateGroup::rent(Player *pLand)
{
	if (m_rentMath.size())
	{
		Estate *estate = pLand->estate();

		std::string rentMath = m_rentMath;
		stringReplace(rentMath, "${DICE}", std::string(itoa((m_game->dice[0] + m_game->dice[1]))));
		stringReplace(rentMath, "${GROUPOWNED}", std::string(itoa(estate->groupSize(estate->owner()))));
		stringReplace(rentMath, "${HOUSES}", std::string(itoa(estate->houses())));

		try {
			std::string exprStr(rentMath);
			std::auto_ptr<math::TNode<double> >expr(math::TReader<double>::parse(exprStr));
			std::auto_ptr<math::TNode<double> >se(math::TSimplifier<double>::simplify(expr.get()));
			return atoi(math::TPrinter<double>::print(se.get()).c_str());
		} catch (const math::EMath& e) {
			return 0;
		} catch (...) {
			return 0;
		}
	}
	return 0;
}
